tree grammar NativeLibraryWalker;

options {
  language = Java;
  tokenVocab = NativeLibrary;
  ASTLabelType = CommonTree;
}

@header {
  package de.timoh.galaxyparser;
  import de.timoh.galaxyparser.typchecking.*;
}

evaluator
	:(function | global)* EOF 
	;
	
function
	: ^(NATIVE i1=IDENTIFIER p=parameter_list) {SymbolTable.putFunction(i1.getText(),new Function(i1.getText(),Type.Void,p));}
	| ^(NATIVE t=type i1=IDENTIFIER p=parameter_list) {SymbolTable.putFunction(i1.getText(),new Function(i1.getText(),t,p));}
	| ^(type IDENTIFIER p=parameter_list)
	; 

parameter_list returns [Parameters p]
	: '(' (p1=type_list)? ')' {p = p1;} 
	;  
	
type_list returns [Parameters p]
@init{
	ArrayList<Type> types = new ArrayList<Type>();
}	
	: (t=type {types.add(t);}','?)+ {Type [] ty = new Type [types.size()];
									for(int i = 0;i < types.size();i++){
										ty[i] = types.get(i);
									}
									p = new Parameters(ty);
									}
	;
global
	: ^(t=type i1=IDENTIFIER) {SymbolTable.putGlobal(i1.getText(),t);}
	;
	
type returns [Type t]
@init{
	t = null;
}
	:	BOOLEAN {t = Type.Boolean;}
	|	BYTE {t = Type.Byte;}
	|	CHAR {t = Type.Char;}
	|	FIXED_LIT {t = Type.Fixed;}
	|	GAMELINK {t = Type.Gamelink;}
	|	COLOR {t = Type.Color;}
	|	INT {t = Type.Integer;}
	|	POINT {t = Type.Point;}
	|	STRING {t = Type.String;}
	|	ABILCMD {t = Type.Abilcmd;}
	|	ACTOR {t = Type.Actor;}
	|	ACTORSCOPE {t = Type.Actorscope;}
	|	BANK {t = Type.Bank;}
	|	CAMERAINFO {t = Type.Camerainfo;}
	|	MARKER {t = Type.Marker;}
	|	ORDER {t = Type.Order;}
	|	PLAYERGROUP {t = Type.Playergroup;}
	|	REGION {t = Type.Region;}
	|	REVEALER {t = Type.Revealer;}
	|	SOUNDLINK {t = Type.Soundlink;}
	|	TEXT {t = Type.Text;}
	|	TIMER {t = Type.Timer;}
	|	TRANSMISSIONSOURCE {t = Type.Transmissionsource;}
	|	TRIGGER {t = Type.Trigger;}
	|	UNITFILTER {t = Type.Unitfilter;}
	|	UNITGROUP {t = Type.Unitgroup;}
	|	UNITREF {t = Type.Unitref;}
	|	UNIT {t = Type.Unit;}
	|	WAVEINFO {t = Type.Waveinfo;}
	|	WAVETARGET {t = Type.Wavetarget;}
	|	DOODAD {t = Type.Doodad;}
	|	AIFILTER {t = Type.AIFilter;}
	|	SOUND {t = Type.Sound;}
	|	WAVE {t = Type.Wave;}
	;
	
